@@ -105,6 +105,71 @@ def _build_cookiejar(cookiefile):
     return retcj
 
 
+class _BugzillaToken(object):
+    def __init__(self, uri, tokenfilename):
+        self.tokenfilename = tokenfilename
+        self.tokenfile = SafeConfigParser()
+        self.domain = urlparse(uri)[1]
+
+        if self.tokenfilename:
+            self.tokenfile.read(self.tokenfilename)
+
+        if self.domain not in self.tokenfile.sections():
+            self.tokenfile.add_section(self.domain)
+
+    @property
+    def value(self):
+        if self.tokenfile.has_option(self.domain, 'token'):
+            return self.tokenfile.get(self.domain, 'token')
+        else:
+            return None
+
+    @value.setter
+    def value(self, value):
+        if self.value == value:
+            return
+
+        if value is None:
+            self.tokenfile.remove_option(self.domain, 'token')
+        else:
+            self.tokenfile.set(self.domain, 'token', value)
+
+        if self.tokenfilename:
+            with open(self.tokenfilename, 'w') as tokenfile:
+                log.debug("Saving to tokenfile")
+                self.tokenfile.write(tokenfile)
+
+    def __repr__(self):
+        return '<Bugzilla Token :: %s>' % (self.value)
+
+
+class _BugzillaServerProxy(ServerProxy):
+    def __init__(self, uri, tokenfile, *args, **kwargs):
+        # pylint: disable=super-init-not-called
+        # No idea why pylint complains here, must be a bug
+        ServerProxy.__init__(self, uri, *args, **kwargs)
+        self.token = _BugzillaToken(uri, tokenfile)
+
+    def clear_token(self):
+        self.token.value = None
+
+    def _ServerProxy__request(self, methodname, params):
+        if self.token.value is not None:
+            if len(params) == 0:
+                params = ({}, )
+
+            if 'Bugzilla_token' not in params[0]:
+                params[0]['Bugzilla_token'] = self.token.value
+
+        # pylint: disable=maybe-no-member
+        ret = ServerProxy._ServerProxy__request(self, methodname, params)
+        # pylint: enable=maybe-no-member
+
+        if isinstance(ret, dict) and 'token' in ret.keys():
+            self.token.value = ret.get('token')
+        return ret
+
+
 class RequestsTransport(Transport):
     user_agent = 'Python/Bugzilla'
 
@@ -128,7 +193,7 @@ class RequestsTransport(Transport):
 
         self.request_defaults = {
             'cert': sslcafile if self.use_https else None,
-            'cookies': cookiejar if cookiejar else None,
+            'cookies': cookiejar,
             'verify': sslverify,
             'headers': {
                 'Content-Type': 'text/xml',
@@ -188,6 +253,10 @@ class RequestsTransport(Transport):
         # xmlrpclib fails to escape \r
         request_body = request_body.replace(b'\r', b'&#xd;')
 
+        # Needed for python-requests < 2.0 with python3, otherwise we get
+        # Content-Type error later for the POST request
+        request_body = request_body.decode('utf-8')
+
         return self._request_helper(url, request_body)
 
 
@@ -196,6 +265,28 @@ class BugzillaError(Exception):
     pass
 
 
+class _FieldAlias(object):
+    """
+    Track API attribute names that differ from what we expose in users.
+
+    For example, originally 'short_desc' was the name of the property that
+    maps to 'summary' on modern bugzilla. We want pre-existing API users
+    to be able to continue to use Bug.short_desc, and
+    query({"short_desc": "foo"}). This class tracks that mapping.
+
+    @oldname: The old attribute name
+    @newname: The modern attribute name
+    @is_api: If True, use this mapping for values sent to the xmlrpc API
+        (like the query example)
+    @is_bug: If True, use this mapping for Bug attribute names.
+    """
+    def __init__(self, newname, oldname, is_api=True, is_bug=True):
+        self.newname = newname
+        self.oldname = oldname
+        self.is_api = is_api
+        self.is_bug = is_bug
+
+
 class BugzillaBase(object):
     '''An object which represents the data and methods exported by a Bugzilla
     instance. Uses xmlrpclib to do its thing. You'll want to create one thusly:
@@ -231,10 +322,14 @@ class BugzillaBase(object):
         to ~/.bugzillacookies.  If set to None, the library won't save the
         cookies persistently.
     '''
+
+    # bugzilla version that the class is targetting. filled in by
+    # subclasses
     bz_ver_major = 0
     bz_ver_minor = 0
 
-    # This is the class API version
+    # Intended to be the API version of the class, but historically is
+    # unused and basically worthless since we don't plan on breaking API.
     version = "0.1"
 
     @staticmethod
@@ -276,7 +371,7 @@ class BugzillaBase(object):
         """
         Turn passed url into a bugzilla XMLRPC web url
         """
-        if not '://' in url:
+        if '://' not in url:
             log.debug('No scheme given for url, assuming https')
             url = 'https://' + url
         if url.count('/') < 3:
@@ -285,7 +380,17 @@ class BugzillaBase(object):
         return url
 
     def __init__(self, url=None, user=None, password=None, cookiefile=-1,
-                 sslverify=True):
+                 sslverify=True, tokenfile=-1):
+        # Hook to allow Bugzilla autodetection without weirdly overriding
+        # __init__
+        if self._init_class_from_url(url, sslverify):
+            kwargs = locals().copy()
+            del(kwargs["self"])
+
+            # pylint: disable=non-parent-init-called
+            self.__class__.__init__(self, **kwargs)
+            return
+
         # Settings the user might want to tweak
         self.user = user or ''
         self.password = password or ''
@@ -296,6 +401,7 @@ class BugzillaBase(object):
         self._sslverify = bool(sslverify)
 
         self.logged_in = False
+        self.bug_autorefresh = True
 
         # Bugzilla object state info that users shouldn't mess with
         self._proxy = None
@@ -307,11 +413,42 @@ class BugzillaBase(object):
 
         if cookiefile == -1:
             cookiefile = os.path.expanduser('~/.bugzillacookies')
+        if tokenfile == -1:
+            tokenfile = os.path.expanduser("~/.bugzillatoken")
+        log.debug("Using tokenfile=%s", tokenfile)
         self.cookiefile = cookiefile
+        self.tokenfile = tokenfile
+
+        # List of field aliases. Maps old style RHBZ parameter
+        # names to actual upstream values. Used for createbug() and
+        # query include_fields at least.
+        self._field_aliases = []
+        self._add_field_alias('summary', 'short_desc')
+        self._add_field_alias('description', 'comment')
+        self._add_field_alias('platform', 'rep_platform')
+        self._add_field_alias('severity', 'bug_severity')
+        self._add_field_alias('status', 'bug_status')
+        self._add_field_alias('id', 'bug_id')
+        self._add_field_alias('blocks', 'blockedby')
+        self._add_field_alias('blocks', 'blocked')
+        self._add_field_alias('depends_on', 'dependson')
+        self._add_field_alias('creator', 'reporter')
+        self._add_field_alias('url', 'bug_file_loc')
+        self._add_field_alias('dupe_of', 'dupe_id')
+        self._add_field_alias('dupe_of', 'dup_id')
+        self._add_field_alias('comments', 'longdescs')
+        self._add_field_alias('creation_time', 'opendate')
+        self._add_field_alias('creation_time', 'creation_ts')
+        self._add_field_alias('whiteboard', 'status_whiteboard')
+        self._add_field_alias('last_change_time', 'delta_ts')
 
         if url:
             self.connect(url)
 
+    def _init_class_from_url(self, url, sslverify):
+        ignore = url
+        ignore = sslverify
+
     def _init_private_data(self):
         '''initialize private variables used by this bugzilla instance.'''
         self._proxy = None
@@ -321,8 +458,8 @@ class BugzillaBase(object):
         self._components_details = {}
 
     def _get_user_agent(self):
-        ret = ('Python-urllib bugzilla.py/%s %s/%s' %
-               (__version__, str(self.__class__.__name__), self.version))
+        ret = ('Python-urllib bugzilla.py/%s %s' %
+               (__version__, str(self.__class__.__name__)))
         return ret
     user_agent = property(_get_user_agent)
 
@@ -362,6 +499,17 @@ class BugzillaBase(object):
                 return p['id']
         raise ValueError('No product named "%s"' % product)
 
+    def _add_field_alias(self, *args, **kwargs):
+        self._field_aliases.append(_FieldAlias(*args, **kwargs))
+
+    def _get_bug_aliases(self):
+        return [(f.newname, f.oldname)
+                for f in self._field_aliases if f.is_bug]
+
+    def _get_api_aliases(self):
+        return [(f.newname, f.oldname)
+                for f in self._field_aliases if f.is_api]
+
 
     ###################
     # Cookie handling #
@@ -439,8 +587,8 @@ class BugzillaBase(object):
         self._transport = RequestsTransport(
             url, self._cookiejar, sslverify=self._sslverify)
         self._transport.user_agent = self.user_agent
-        self._proxy = ServerProxy(url, self._transport)
-
+        self._proxy = _BugzillaServerProxy(url, self.tokenfile,
+            self._transport)
 
         self.url = url
         # we've changed URLs - reload config
@@ -474,7 +622,8 @@ class BugzillaBase(object):
         logged_in attribute to True, if successful.
 
         If user is not set, the value of Bugzilla.user will be used. If *that*
-        is not set, ValueError will be raised.
+        is not set, ValueError will be raised. If login fails, BugzillaError
+        will be raised.
 
         This method will be called implicitly at the end of connect() if user
         and password are both set. So under most circumstances you won't need
@@ -491,14 +640,14 @@ class BugzillaBase(object):
             raise ValueError("missing password")
 
         try:
-            r = self._login(self.user, self.password)
+            ret = self._login(self.user, self.password)
             self.logged_in = True
-            log.info("login successful - dropping password from memory")
             self.password = ''
+            log.info("login successful for user=%s", self.user)
+            return ret
         except Fault:
-            r = False
-
-        return r
+            e = sys.exc_info()[1]
+            raise BugzillaError("Login failed: %s" % str(e.faultString))
 
     def logout(self):
         '''Log out of bugzilla. Drops server connection and user info, and
@@ -515,8 +664,8 @@ class BugzillaBase(object):
     #############################################
 
     def _getbugfields(self):
-        '''IMPLEMENT ME: Get bugfields from Bugzilla.'''
-        raise NotImplementedError
+        raise RuntimeError("This bugzilla version does not support listing "
+            "bug fields.")
 
     def getbugfields(self, force_refresh=False):
         '''
@@ -760,35 +909,11 @@ class BugzillaBase(object):
     #
     # As of Dec 2012 it seems like only RH bugzilla actually has behavior
     # like this, for upstream bz it returns all info for every Bug.get()
-    getbug_extra_fields = []
+    _getbug_extra_fields = []
+    _supports_getbug_extra_fields = False
 
-    # List of field aliases. Maps old style RHBZ parameter names to actual
-    # upstream values. Used for createbug() and query include_fields at
-    # least.
-    #
-    # Format is (currentname, oldname)
-    field_aliases = (
-        ('summary', 'short_desc'),
-        ('description', 'comment'),
-        ('platform', 'rep_platform'),
-        ('severity', 'bug_severity'),
-        ('status', 'bug_status'),
-        ('id', 'bug_id'),
-        ('blocks', 'blockedby'),
-        ('blocks', 'blocked'),
-        ('depends_on', 'dependson'),
-        ('creator', 'reporter'),
-        ('url', 'bug_file_loc'),
-        ('dupe_of', 'dupe_id'),
-        ('dupe_of', 'dup_id'),
-        ('comments', 'longdescs'),
-        ('creation_time', 'opendate'),
-        ('creation_time', 'creation_ts'),
-        ('whiteboard', 'status_whiteboard'),
-        ('last_change_time', 'delta_ts'),
-    )
-
-    def _getbugs(self, idlist, simple=False, permissive=True):
+    def _getbugs(self, idlist, simple=False, permissive=True,
+            include_fields=None, exclude_fields=None, extra_fields=None):
         '''
         Return a list of dicts of full bug info for each given bug id.
         bug ids that couldn't be found will return None instead of a dict.
@@ -804,11 +929,20 @@ class BugzillaBase(object):
                 # String aliases can be passed as well
                 idlist.append(i)
 
+        extra_fields = self._listify(extra_fields or [])
+        if not simple:
+            extra_fields += self._getbug_extra_fields
+
         getbugdata = {"ids": idlist}
         if permissive:
             getbugdata["permissive"] = 1
-        if self.getbug_extra_fields and not simple:
-            getbugdata["extra_fields"] = self.getbug_extra_fields
+        if self.bz_ver_major >= 4:
+            if include_fields:
+                getbugdata["include_fields"] = self._listify(include_fields)
+            if exclude_fields:
+                getbugdata["exclude_fields"] = self._listify(exclude_fields)
+        if self._supports_getbug_extra_fields:
+            getbugdata["extra_fields"] = extra_fields
 
         log.debug("Calling Bug.get_bugs with: %s", getbugdata)
         r = self._proxy.Bug.get_bugs(getbugdata)
@@ -834,21 +968,31 @@ class BugzillaBase(object):
 
         return ret
 
-    def _getbug(self, objid, simple=False):
+    def _getbug(self, objid, simple=False,
+            include_fields=None, exclude_fields=None, extra_fields=None):
         '''Return a dict of full bug info for the given bug id'''
-        return self._getbugs([objid], simple=simple, permissive=False)[0]
+        return self._getbugs([objid], simple=simple, permissive=False,
+            include_fields=include_fields, exclude_fields=exclude_fields,
+            extra_fields=extra_fields)[0]
 
-    def getbug(self, objid):
+    def getbug(self, objid,
+            include_fields=None, exclude_fields=None, extra_fields=None):
         '''Return a Bug object with the full complement of bug data
         already loaded.'''
-        return _Bug(bugzilla=self, dict=self._getbug(objid))
+        data = self._getbug(objid, include_fields=include_fields,
+            exclude_fields=exclude_fields, extra_fields=extra_fields)
+        return _Bug(self, dict=data, autorefresh=self.bug_autorefresh)
 
-    def getbugs(self, idlist):
+    def getbugs(self, idlist,
+        include_fields=None, exclude_fields=None, extra_fields=None):
         '''Return a list of Bug objects with the full complement of bug data
         already loaded. If there's a problem getting the data for a given id,
         the corresponding item in the returned list will be None.'''
-        return [(b and _Bug(bugzilla=self, dict=b)) or None
-                for b in self._getbugs(idlist)]
+        data = self._getbugs(idlist, include_fields=include_fields,
+            exclude_fields=exclude_fields, extra_fields=extra_fields)
+        return [(b and _Bug(self, dict=b,
+                            autorefresh=self.bug_autorefresh)) or None
+                for b in data]
 
     # Since for so long getbugsimple was just getbug, I don't think we can
     # remove any fields without possibly causing a slowdown for some
@@ -856,14 +1000,17 @@ class BugzillaBase(object):
     # big stuff'
     def getbugsimple(self, objid):
         '''Return a Bug object given bug id, populated with simple info'''
-        return _Bug(bugzilla=self, dict=self._getbug(objid, simple=True))
+        return _Bug(self,
+                    dict=self._getbug(objid, simple=True),
+                    autorefresh=self.bug_autorefresh)
 
     def getbugssimple(self, idlist):
         '''Return a list of Bug objects for the given bug ids, populated with
         simple info. As with getbugs(), if there's a problem getting the data
         for a given bug ID, the corresponding item in the returned list will
         be None.'''
-        return [(b and _Bug(bugzilla=self, dict=b)) or None
+        return [(b and _Bug(self, dict=b,
+                autorefresh=self.bug_autorefresh)) or None
                 for b in self._getbugs(idlist, simple=True)]
 
 
@@ -871,6 +1018,16 @@ class BugzillaBase(object):
     # query methods #
     #################
 
+    def _convert_include_field_list(self, _in):
+        if not _in:
+            return _in
+
+        for newname, oldname in self._get_api_aliases():
+            if oldname in _in:
+                _in.remove(oldname)
+                if newname not in _in:
+                    _in.append(newname)
+        return _in
 
     def build_query(self,
                     product=None,
@@ -908,7 +1065,8 @@ class BugzillaBase(object):
                     quicksearch=None,
                     savedsearch=None,
                     savedsearch_sharer_id=None,
-                    sub_component=None):
+                    sub_component=None,
+                    tags=None):
         """
         Build a query string from passed arguments. Will handle
         query parameter differences between various bugzilla versions.
@@ -922,9 +1080,9 @@ class BugzillaBase(object):
 
         Then pass the output to Bugzilla.query()
         """
-        ignore = include_fields
         ignore = emailtype
         ignore = booleantype
+        ignore = include_fields
 
         for key, val in [
             ('fixed_in', fixed_in),
@@ -941,7 +1099,7 @@ class BugzillaBase(object):
             ('sharer_id', savedsearch_sharer_id),
             ('sub_component', sub_component),
         ]:
-            if not val is None:
+            if val is not None:
                 raise RuntimeError("'%s' search not supported by this "
                                    "bugzilla" % key)
 
@@ -966,6 +1124,7 @@ class BugzillaBase(object):
             "cc": cc,
             "qa_contact": qa_contact,
             "reporter": reporter,
+            "tag": self._listify(tags),
         }
 
         # Strip out None elements in the dict
@@ -990,7 +1149,8 @@ class BugzillaBase(object):
         '''
         r = self._query(query)
         log.debug("Query returned %s bugs", len(r['bugs']))
-        return [_Bug(bugzilla=self, dict=b) for b in r['bugs']]
+        return [_Bug(self, dict=b,
+                autorefresh=self.bug_autorefresh) for b in r['bugs']]
 
     def simplequery(self, product, version='', component='',
                     string='', matchtype='allwordssubstr'):
@@ -1061,6 +1221,24 @@ class BugzillaBase(object):
         log.debug("Calling Flag.update with: %s", d)
         return self._proxy.Flag.update(d)
 
+    def update_tags(self, idlist, tags_add=None, tags_remove=None):
+        '''
+        Updates the 'tags' field for a bug.
+        '''
+        tags = {}
+        if tags_add:
+            tags["add"] = self._listify(tags_add)
+        if tags_remove:
+            tags["remove"] = self._listify(tags_remove)
+
+        d = {
+            "ids": self._listify(idlist),
+            "tags": tags,
+        }
+
+        log.debug("Calling Bug.update_tags with: %s", d)
+        return self._proxy.Bug.update_tags(d)
+
 
     def build_update(self,
                      alias=None,
@@ -1393,14 +1571,7 @@ class BugzillaBase(object):
         ret.update(localdict)
         return ret
 
-
-    def createbug(self, *args, **kwargs):
-        '''
-        Create a bug with the given info. Returns a new Bug object.
-        Check bugzilla API documentation for valid values, at least
-        product, component, summary, version, and description need to
-        be passed.
-        '''
+    def _validate_createbug(self, *args, **kwargs):
         # Previous API required users specifying keyword args that mapped
         # to the XMLRPC arg names. Maintain that bad compat, but also allow
         # receiving a single dictionary like query() does
@@ -1418,19 +1589,30 @@ class BugzillaBase(object):
 
         # If we're getting a call that uses an old fieldname, convert it to the
         # new fieldname instead.
-        for newfield, oldfield in self.field_aliases:
-            if (newfield in self.createbug_required and
-                newfield not in data and
-                oldfield in data):
-                data[newfield] = data.pop(oldfield)
+        for newname, oldname in self._get_api_aliases():
+            if (newname in self.createbug_required and
+                newname not in data and
+                oldname in data):
+                data[newname] = data.pop(oldname)
 
         # Back compat handling for check_args
         if "check_args" in data:
             del(data["check_args"])
 
+        return data
+
+    def createbug(self, *args, **kwargs):
+        '''
+        Create a bug with the given info. Returns a new Bug object.
+        Check bugzilla API documentation for valid values, at least
+        product, component, summary, version, and description need to
+        be passed.
+        '''
+        data = self._validate_createbug(*args, **kwargs)
         log.debug("Calling Bug.create with: %s", data)
         rawbug = self._proxy.Bug.create(data)
-        return _Bug(self, bug_id=rawbug["id"])
+        return _Bug(self, bug_id=rawbug["id"],
+                    autorefresh=self.bug_autorefresh)
 
 
     ##############################
@@ -1573,7 +1755,8 @@ class BugzillaBase(object):
         self.createuser(user, name)
 
     def getqueryinfo(self, force_refresh=False):
-        raise NotImplementedError("getqueryinfo is deprecated and the "
-                "information is not provided by any modern bugzilla.")
+        ignore = force_refresh
+        raise RuntimeError("getqueryinfo is deprecated and the "
+            "information is not provided by any modern bugzilla.")
     querydata = property(getqueryinfo)
     querydefaults = property(getqueryinfo)
