@@ -16,7 +16,7 @@ from bugzilla.bugzilla4 import Bugzilla4
 import tests
 
 
-bz4 = Bugzilla4(cookiefile=None)
+bz4 = Bugzilla4(cookiefile=None, tokenfile=None)
 
 
 class CreatebugTest(unittest.TestCase):
@@ -75,3 +75,12 @@ class CreatebugTest(unittest.TestCase):
             "--keywords TestOnly --keywords ZStream",
             out
         )
+
+    def testFieldConversion(self):
+        vc = self.bz._validate_createbug  # pylint: disable=protected-access
+        out = vc(product="foo", component="bar",
+            version="12", description="foo", short_desc="bar",
+            check_args=False)
+        self.assertDictEqual(out,
+            {'component': 'bar', 'description': 'foo', 'product': 'foo',
+             'summary': 'bar', 'version': '12'})
