@@ -25,10 +25,12 @@ else:
 
 import bugzilla
 from bugzilla import Bugzilla
+from bugzilla.base import _BugzillaToken
 
 import tests
 
 cf = os.path.expanduser("~/.bugzillacookies")
+tf = os.path.expanduser("~/.bugzillatoken")
 
 
 def _split_int(s):
@@ -40,19 +42,23 @@ class BaseTest(unittest.TestCase):
     bzclass = None
 
     def _testBZClass(self):
-        bz = Bugzilla(url=self.url, cookiefile=None)
+        bz = Bugzilla(url=self.url, cookiefile=None, tokenfile=None)
         self.assertTrue(bz.__class__ is self.bzclass)
 
-    def _testCookie(self):
-        cookiefile = cf
+    def _testCookieOrToken(self):
         domain = urlparse(self.url)[1]
-        if os.path.exists(cookiefile):
-            out = open(cookiefile).read(1024)
+        if os.path.exists(cf):
+            out = open(cf).read(1024)
             if domain in out:
                 return
 
-        raise RuntimeError("%s must exist and contain domain '%s'" %
-                           (cookiefile, domain))
+        if os.path.exists(tf):
+            token = _BugzillaToken(self.url, tokenfilename=tf)
+            if token.value is not None:
+                return
+
+        raise RuntimeError("%s or %s must exist and contain domain '%s'" %
+                           (cf, tf, domain))
 
 
 class RHPartnerTest(BaseTest):
@@ -60,7 +66,7 @@ class RHPartnerTest(BaseTest):
     # doesn't send out emails and is blown away occasionally. The front
     # page has some info.
     url = "https://partner-bugzilla.redhat.com/xmlrpc.cgi"
-    #url = "https://bzweb01-devel.app.eng.rdu.redhat.com/"
+    # url = "https://bzweb01-devel.app.eng.rdu.redhat.com/"
     bzclass = bugzilla.RHBugzilla
 
 
@@ -73,14 +79,14 @@ class RHPartnerTest(BaseTest):
             print("\nNo admin privs, skipping %s" % funcname)
         return ret
 
-    def _check_rh_privs(self, bz, funcname, quiet=False):
+    def _check_rh_privs(self, bz, funcname, authtype, quiet=False):
         noprivs = bool(bz.getbugs([184858]) == [None])
         if noprivs and not quiet:
-            print("\nNo RH privs, skipping %s" % funcname)
+            print("\nNo RH %s privs, skipping %s" % (authtype, funcname))
         return not noprivs
 
 
-    test1 = BaseTest._testCookie
+    test1 = BaseTest._testCookieOrToken
     test2 = BaseTest._testBZClass
 
 
@@ -88,7 +94,7 @@ class RHPartnerTest(BaseTest):
         """
         Create a bug with minimal amount of fields, then close it
         """
-        bz = self.bzclass(url=self.url, cookiefile=cf)
+        bz = self.bzclass(url=self.url, cookiefile=cf, tokenfile=tf)
         component = "python-bugzilla"
         version = "rawhide"
         summary = ("python-bugzilla test basic bug %s" %
@@ -126,7 +132,7 @@ class RHPartnerTest(BaseTest):
         """
         Create a bug using all 'new' fields, check some values, close it
         """
-        bz = self.bzclass(url=self.url, cookiefile=cf)
+        bz = self.bzclass(url=self.url, cookiefile=cf, tokenfile=tf)
 
         summary = ("python-bugzilla test manyfields bug %s" %
                    datetime.datetime.today())
@@ -175,7 +181,7 @@ class RHPartnerTest(BaseTest):
         """
         Modify status and comment fields for an existing bug
         """
-        bz = self.bzclass(url=self.url, cookiefile=cf)
+        bz = self.bzclass(url=self.url, cookiefile=cf, tokenfile=tf)
         bugid = "663674"
         cmd = "bugzilla modify %s " % bugid
 
@@ -244,7 +250,7 @@ class RHPartnerTest(BaseTest):
         """
         Modify cc, assignee, qa_contact for existing bug
         """
-        bz = self.bzclass(url=self.url, cookiefile=cf)
+        bz = self.bzclass(url=self.url, cookiefile=cf, tokenfile=tf)
         bugid = "663674"
         cmd = "bugzilla modify %s " % bugid
 
@@ -291,7 +297,7 @@ class RHPartnerTest(BaseTest):
         """
         Modify flags and fixed_in for 2 bugs
         """
-        bz = self.bzclass(url=self.url, cookiefile=cf)
+        bz = self.bzclass(url=self.url, cookiefile=cf, tokenfile=tf)
         bugid1 = "461686"
         bugid2 = "461687"
         cmd = "bugzilla modify %s %s " % (bugid1, bugid2)
@@ -367,7 +373,7 @@ class RHPartnerTest(BaseTest):
     def test7ModifyMisc(self):
         bugid = "461686"
         cmd = "bugzilla modify %s " % bugid
-        bz = self.bzclass(url=self.url, cookiefile=cf)
+        bz = self.bzclass(url=self.url, cookiefile=cf, tokenfile=tf)
         bug = bz.getbug(bugid)
 
         # modify --dependson
@@ -461,7 +467,7 @@ class RHPartnerTest(BaseTest):
         """
         Get and set attachments for a bug
         """
-        bz = self.bzclass(url=self.url, cookiefile=cf)
+        bz = self.bzclass(url=self.url, cookiefile=cf, tokenfile=tf)
         getallbugid = "663674"
         setbugid = "461686"
         cmd = "bugzilla attach "
@@ -537,7 +543,7 @@ class RHPartnerTest(BaseTest):
 
 
     def test9Whiteboards(self):
-        bz = self.bzclass(url=self.url, cookiefile=cf)
+        bz = self.bzclass(url=self.url, cookiefile=cf, tokenfile=tf)
         bug_id = "663674"
         cmd = "bugzilla modify %s " % bug_id
         bug = bz.getbug(bug_id)
@@ -606,16 +612,17 @@ class RHPartnerTest(BaseTest):
                             "--user foobar@example.com "
                             "--password foobar login" % self.url, None,
                             expectfail=True)
-        self.assertTrue("Logging in... failed." in ret)
+        self.assertTrue("Logging in... Login failed:" in ret)
 
 
     def test11UserUpdate(self):
         # This won't work if run by the same user we are using
-        bz = self.bzclass(url=self.url, cookiefile=cf)
+        bz = self.bzclass(url=self.url, cookiefile=cf, tokenfile=tf)
         email = "anaconda-maint-list@redhat.com"
         group = "fedora_contrib"
 
-        if not self._check_have_admin(bz, sys._getframe().f_code.co_name):
+        fn = sys._getframe().f_code.co_name  # pylint: disable=protected-access
+        if not self._check_have_admin(bz, fn):
             return
 
         user = bz.getuser(email)
@@ -646,7 +653,7 @@ class RHPartnerTest(BaseTest):
 
 
     def test11ComponentEditing(self):
-        bz = self.bzclass(url=self.url, cookiefile=cf)
+        bz = self.bzclass(url=self.url, cookiefile=cf, tokenfile=tf)
         component = ("python-bugzilla-testcomponent-%s" %
                      str(random.randint(1, 1024 * 1024 * 1024)))
         basedata = {
@@ -654,12 +661,14 @@ class RHPartnerTest(BaseTest):
             "component": component,
         }
 
-        if not self._check_have_admin(bz, sys._getframe().f_code.co_name):
+        fn = sys._getframe().f_code.co_name  # pylint: disable=protected-access
+        if not self._check_have_admin(bz, fn):
             return
 
 
         def compare(data, newid):
-            products = bz._proxy.Product.get({"names": [basedata["product"]]})
+            proxy = bz._proxy  # pylint: disable=protected-access
+            products = proxy.Product.get({"names": [basedata["product"]]})
             compdata = None
             for c in products["products"][0]["components"]:
                 if int(c["id"]) == int(newid):
@@ -699,8 +708,11 @@ class RHPartnerTest(BaseTest):
         compare(data, newid)
 
     def test12SetCookie(self):
-        bz = self.bzclass(url=self.url, cookiefile=cf)
-        if not self._check_rh_privs(bz, sys._getframe().f_code.co_name):
+        bz = self.bzclass("partner-bugzilla.redhat.com",
+            cookiefile=cf, tokenfile=None)
+
+        fn = sys._getframe().f_code.co_name  # pylint: disable=protected-access
+        if not self._check_rh_privs(bz, "cookie", fn):
             return
 
         try:
@@ -714,10 +726,11 @@ class RHPartnerTest(BaseTest):
         bz.disconnect()
         bz.cookiefile = None
         bz.connect()
-        self.assertFalse(bool(self._check_rh_privs(bz, "", True)))
+        self.assertFalse(bool(self._check_rh_privs(
+            bz, "", "cookie", quiet=True)))
 
     def test13SubComponents(self):
-        bz = self.bzclass(url=self.url, cookiefile=cf)
+        bz = self.bzclass(url=self.url, cookiefile=cf, tokenfile=tf)
         # Long closed RHEL5 lvm2 bug. This component has sub_components
         bug = bz.getbug("185526")
         self.assertEquals(bug.component, "lvm2")
